
#include "DirectoryLister.h"
#include "stringhelper.h"
#include "databasewrapper.h"
#include "sqlite3/sqlite3.h"
#include <string>
using namespace std;

TagDirectoryLister::TagDirectoryLister(std::string startDir) {
   //split directories to get tags already in the list

   std::string searchSql = "";
   bool firstQuery = true;

   //SELECT fileId from tag_album WHERE value='Das Zweite Kapitel' INTERSECT SELECT fileId from tag_title
   StringTokenizer<std::string> strTok(startDir);
   std::string token = strTok("\\");
   while(!token.empty()) {
      std::string::size_type equalPos = token.find_first_of("=");
      std::string valueToFind;
      if(equalPos != std::string::npos) {          //split "tag=test"
         valueToFind = token.substr(equalPos+1);
         token = token.substr(0, equalPos);
      }
      if(!firstQuery) {
         searchSql += " INTERSECT ";
      } else {
         firstQuery = false;
      }
      searchSql += "SELECT fileId from tag_";
      searchSql += token;
      if(!valueToFind.empty()) {
         searchSql += " WHERE value='";
         searchSql += valueToFind;
         searchSql += "'";
      }
      mTagsInDirectoryPath.insert(pair<std::string, std::string>(token, token));

      token = strTok("\\");
   }
   if(searchSql.length() == 0) {
      searchSql = "SELECT key from files";
   }

   sqlite3_prepare(db, searchSql.c_str(), searchSql.length(), &query, NULL);
}

TagDirectoryLister::~TagDirectoryLister() {
   sqlite3_finalize(query);
}

std::string TagDirectoryLister::getNextFile() {
   std::string filename;
   if(sqlite3_step(query) == SQLITE_ROW) {
      sqlite3_stmt *fileQuery;
      std::string fileQueryStr;
      fileQueryStr = format("SELECT * from files WHERE key=%i", sqlite3_column_int(query, 0));
      sqlite3_prepare(db, fileQueryStr.c_str(), fileQueryStr.length(), &fileQuery, NULL);

      if(sqlite3_step(fileQuery) == SQLITE_ROW) {
         //sqlite3_exec(db, "CREATE TABLE files (key INTEGER PRIMARY KEY,filename TEXT,path TEXT,tags TEXT);", NULL, NULL, &errmsg);
         filename = (const char*)sqlite3_column_text(fileQuery, 1);
         std::string tags = (const char*)sqlite3_column_text(fileQuery, 3);
         addTagToAlltagsInCurDirectory(tags);
      }
      sqlite3_finalize(fileQuery);
   }
   return filename;
}

void TagDirectoryLister::addTagToAlltagsInCurDirectory(std::string allTags) {
   StringTokenizer<std::string> strTok(allTags);
   std::string token = strTok(",");
   while(!token.empty()) {
      if(token.compare("(null)")!=0) {
         mTags.insert(pair<std::string, std::string>(token, token));
      }
      token = strTok(",");
   }
}


std::string TagDirectoryLister::getFirstTagInCurDirectory() {
   mTagIterator = mTags.begin();
   return getNextTagInCurDirectory();
}

std::string TagDirectoryLister::getNextTagInCurDirectory() {
   std::string result;
   if(mTagIterator != mTags.end()) {         //already at the end of the list?
      while(mTagIterator != mTags.end() && mTagsInDirectoryPath.count(mTagIterator->first) != 0)
         ++mTagIterator;
      if(mTagIterator != mTags.end()) {      //scanned till end of list?
         result = mTagIterator->first;
         ++mTagIterator;
      }
   }
   return result;
}
