/*  _______         __              
   |_     _|.-----.|  |_.-----.----.
    _|   |_ |     ||   _|  -__|   _|
   |_______||__|__||____|_____|__|  
       coded by Questor / Inter      */

/* Copyright (c) 2007 Kai Jourdan. All rights reserved. 
 * Use of the code is allowed under either the Artistic
 * License 2.0 terms or the GNU General Public License.*/

#include "console.h"
using namespace istl;

#include <windows.h>
#include <io.h>
#include <fcntl.h>
#include <stdio.h>

console::console(const char *title) {
   if(AllocConsole()) {
      int handle = _open_osfhandle((intptr_t)GetStdHandle(STD_OUTPUT_HANDLE), _O_TEXT);
      FILE *handleFile = _fdopen(handle, "w");
      *stdout = *handleFile;
      int i=setvbuf(stdout, NULL, _IONBF, 0);

      handle = _open_osfhandle((intptr_t)GetStdHandle(STD_ERROR_HANDLE), _O_TEXT);
      handleFile = _fdopen(handle, "w");
      *stderr = *handleFile;
      i = setvbuf(stdout, NULL, _IONBF, 0);
   }
   SetConsoleTitleA(title);
}

console::~console() {
   FreeConsole();
}

void console::setColor(int color) {
   HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
   SetConsoleTextAttribute(hConsole, color);
}

void console::clear() {
   COORD coordScreen = { 0, 0 };
   DWORD cCharsWritten;
   CONSOLE_SCREEN_BUFFER_INFO csbi;
   DWORD dwConSize;
   HANDLE hConsole = GetStdHandle ( STD_OUTPUT_HANDLE );

   GetConsoleScreenBufferInfo ( hConsole, &csbi );
   dwConSize = csbi.dwSize.X * csbi.dwSize.Y;
   FillConsoleOutputCharacter ( hConsole, TEXT ( ' ' ), dwConSize,
                                coordScreen, &cCharsWritten );
   GetConsoleScreenBufferInfo ( hConsole, &csbi );
   FillConsoleOutputAttribute ( hConsole, csbi.wAttributes, dwConSize,
                                coordScreen, &cCharsWritten );
   SetConsoleCursorPosition ( hConsole, coordScreen );
}

void console::setCursor(int posX, int posY) {
   COORD coord;
   coord.X = posX;
   coord.Y = posY;
   HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
   SetConsoleCursorPosition(hConsole, coord);   
}

void console::moveCursorUp(int numberLines) {
   HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
   CONSOLE_SCREEN_BUFFER_INFO screenInfo;
   GetConsoleScreenBufferInfo(hConsole, &screenInfo);
   screenInfo.dwCursorPosition.Y -= numberLines;
   SetConsoleCursorPosition(hConsole, screenInfo.dwCursorPosition);
}

void console::moveCursorDown(int numberLines) {
   HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
   CONSOLE_SCREEN_BUFFER_INFO screenInfo;
   GetConsoleScreenBufferInfo(hConsole, &screenInfo);
   screenInfo.dwCursorPosition.Y += numberLines;
   SetConsoleCursorPosition(hConsole, screenInfo.dwCursorPosition);
}

void console::moveCursorLeft(int numberChars) {
   HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
   CONSOLE_SCREEN_BUFFER_INFO screenInfo;
   GetConsoleScreenBufferInfo(hConsole, &screenInfo);
   screenInfo.dwCursorPosition.X -= numberChars;
   SetConsoleCursorPosition(hConsole, screenInfo.dwCursorPosition);
}

void console::moveCursorRight(int numberChars) {
   HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
   CONSOLE_SCREEN_BUFFER_INFO screenInfo;
   GetConsoleScreenBufferInfo(hConsole, &screenInfo);
   screenInfo.dwCursorPosition.X += numberChars;
   SetConsoleCursorPosition(hConsole, screenInfo.dwCursorPosition);
}

void console::flush() {
   fflush(stdin);
   fflush(stdout);
}

void __cdecl console::printf(const char *buffer, ...) {
   static char textbuffer[2048];
   va_list args;
   va_start(args, buffer);
   vsprintf(textbuffer, buffer, args);
   va_end(args);
   ::printf(textbuffer);
}

int __cdecl console::scanf(const char *buffer, ...) {
   va_list args;
   va_start(args, buffer);
   int res = ::scanf(buffer, args);
   va_end(args);
   return res;
}
