/*  _______         __              
   |_     _|.-----.|  |_.-----.----.
    _|   |_ |     ||   _|  -__|   _|
   |_______||__|__||____|_____|__|  
       coded by Questor / Inter      */

/* Copyright (c) 2007 Kai Jourdan. All rights reserved. 
 * Use of the code is allowed under either the Artistic
 * License 2.0 terms or the GNU General Public License.*/

/**
  * \file console.h
  * \author Kai Jourdan
  */
#ifndef __ISTL_CONSOLE_H__
#define __ISTL_CONSOLE_H__

namespace istl {

class console {
public:
   typedef enum {
      FG_Blue = 0x0001,               /*!< Blue Foreground */
      FG_Green = 0x0002,              /*!< Green Foreground */
      FG_Red = 0x0004,                /*!< Red Foreground */
      FG_Intensity = 0x0008,          /*!< Foreground intensity (makes the foreground colour a shade brighter) */
      BG_Blue = 0x0010,               /*!< Blue Background */
      BG_Green = 0x0020,              /*!< Green Background */
      BG_Red = 0x0040,                /*!< Red Background */
      BG_Intensity = 0x0080,          /*!< Background intensity (makes the foreground colour a shade brighter) */
      FG_Brown = 0x0000,                                      /*!< Brown Foreground (POSIX only) */
      FG_Magenta = FG_Blue | FG_Red,                          /*!< Magenta Foreground */
      FG_Cyan = FG_Blue | FG_Green,                           /*!< Cyan Foreground */
      BG_Brown = 0x0000,                                      /*!< Brown Background (POSIX only) */
      BG_Magenta = BG_Blue | BG_Red,                          /*!< Magenta Background */
      BG_Cyan = BG_Green | BG_Blue,                           /*!< Cyan Background */
      FG_Gray = FG_Blue | FG_Green | FG_Red,                  /*!< Gray Foreground */
      FG_White = FG_Blue | FG_Green | FG_Red | FG_Intensity,  /*!< White Foreground */
      BG_Gray = BG_Blue | BG_Green | BG_Red,                  /*!< Gray Background */
      BG_White = BG_Blue | BG_Green | BG_Red | BG_Intensity   /*!< White Background */
   } Color;

   console(const char *title);
   ~console();

   void setCursor(int posX, int posY);
   void moveCursorUp(int numberLines = 1);
   void moveCursorDown(int numberLines = 1);
   void moveCursorLeft(int numberChars = 1);
   void moveCursorRight(int numberChars = 1);

   void setColor(int color);
   void clear();
   void flush();

   void __cdecl printf(const char *buffer, ...);
   int __cdecl scanf(const char *buffer, ...);
};

}  //namespace istl

#endif
