/*  _______         __              
   |_     _|.-----.|  |_.-----.----.
    _|   |_ |     ||   _|  -__|   _|
   |_______||__|__||____|_____|__|  
       coded by Questor / Inter      */

/* Copyright (c) 2007 Kai Jourdan. All rights reserved. 
 * Use of the code is allowed under either the Artistic
 * License 2.0 terms or the GNU General Public License.*/

#include "console.h"
#include "../sqlite3/sqlite3.h"
#include "../stringhelper.h"
#include "../databasewrapper.h"

#include <string>
#include <sstream>
#include <string.h>
#include <stdio.h>

istl::console *console;
DatabaseWrapper *dbWrapper;

int fileStartDisplayPosition = 0;
int fileSelectorPosition = 0;
int numberFiles;

void clearDisplay() {
   console->clear();
}

void displayFiles() {
//   sqlite3_exec(db, "CREATE TABLE files (key INTEGER PRIMARY KEY,filename TEXT,path TEXT,tags TEXT)", NULL, NULL, &errmsg);
   char *errmsg = 0;

   console->printf("Files:\n");

   sqlite3_stmt *fileQuery;
   char query[] = "SELECT * FROM files;";
   sqlite3_prepare(db, query, strlen(query), &fileQuery, NULL);

   int i=0;
   while(i<fileStartDisplayPosition) {
      sqlite3_step(fileQuery);
   }

   while(sqlite3_step(fileQuery) == SQLITE_ROW) {
      const unsigned char *filename, *tags;
      int id;

      id = sqlite3_column_int(fileQuery, 0);    //id

      filename = sqlite3_column_text(fileQuery, 1);   //filename
      if(filename) {
         console->printf("(%d) %s\n", id, filename, tags);
      }

      tags = sqlite3_column_text(fileQuery, 3);       //tags

      std::string tagsStr;
      tagsStr = (const char*)tags;
      StringTokenizer<std::string> strTok(tagsStr);
      std::string token = strTok(",");
      while(!token.empty()) {
         if(token.compare("(null)") != 0) {
            sqlite3_stmt *tagQuery;
            std::string tagQueryCmd;

            std::stringstream ssStream;
            ssStream << id;

            tagQueryCmd = "SELECT value FROM tag_" + token + " WHERE fileId=" + ssStream.str() + ";";
            sqlite3_prepare(db, tagQueryCmd.c_str(), tagQueryCmd.length(), &tagQuery, NULL);
            while(sqlite3_step(tagQuery) == SQLITE_ROW) {
               const unsigned char *tagValue;
               tagValue = sqlite3_column_text(tagQuery, 0);
               printf("   %s '%s'\n", token.c_str(), tagValue);
            }
            sqlite3_finalize(tagQuery);
         }
         token = strTok(",");
      }
   }
   sqlite3_finalize(fileQuery);
}

void displayTags() {
   char *errmsg = 0;

   console->printf("Tags:\n");

   sqlite3_stmt *fileQuery;
   char query[] = "SELECT name FROM sqlite_master WHERE type='table';";
   sqlite3_prepare(db, query, strlen(query), &fileQuery, NULL);

   int counter = 0;
   while(sqlite3_step(fileQuery) == SQLITE_ROW) {
      const char *hold;
      hold = (const char*)sqlite3_column_text(fileQuery, 0);   //table_name
      if(hold) {
         if(strncmp("tag_", hold, 4) == 0) {
            console->printf("(%d)%s ", counter, hold+4);
            counter++;
         }
      }
   }
   sqlite3_finalize(fileQuery);

   console->printf("\n");
}

void displayMenu() {
   console->printf("\n\n");
   console->printf("1 - Add Tag\n");
   console->printf("2 - Remove Tag\n");
   console->printf("3 - Add Tag to File\n");
   console->printf("4 - Clear all Tags from File\n");
   console->printf("5 - Clear all Tables and recreate with mp3's\n");
   console->printf("Q - Quit\n");
}

void handleKey(char key) {
   switch(key) {
   case '1':   //Add Tag
      {
         printf("Insert Tag Name: ");
         char newTagName[255];
         scanf("%s", newTagName);
         dbWrapper->createTag(newTagName);
      }
      break;
   case '2':   //Remove Tag
      console->printf("not implemented yet!");
      break;
   case '3':   //Add Tag to File
      {
         printf("Insert File Id: ");
         int fileId;
         scanf("%d", &fileId);

         printf("\nInsert Tag Name: ");
         char tag[255];
         scanf("%s", &tag);

         printf("\nInsert Value for Tag: ");
         char value[255];
         scanf("%s", value);

         dbWrapper->insertValueAndFileIntoTag(tag, fileId, value);
      }
      break;
   case '4':   //Clear all Tags from File
      console->printf("not implemented yet!");
      break;
   case '5':
      dbWrapper->createTablesAndInitFromMP3();
      break;

   default:
      break;
   }
}

void main() {
   console = new istl::console("MetaFS Database Editor");
   dbWrapper = new DatabaseWrapper();

   clearDisplay();

   char mainSelection = 0;
   while(mainSelection != 'q') {
      displayFiles();
      displayTags();
      displayMenu();
      scanf("%c", &mainSelection);
      if((mainSelection >= 'A') && (mainSelection <= 'Z'))
         mainSelection |= 0x20;
      handleKey(mainSelection);
   }

   delete dbWrapper;
   delete console;
}
